"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.preprocessReadme = void 0;
var compiler_1 = require("svelte/compiler");
var markdown_it_1 = __importDefault(require("markdown-it"));
var prettier_1 = __importDefault(require("prettier"));
var prismjs_1 = __importDefault(require("prismjs"));
require("prismjs/components/prism-bash");
require("prism-svelte");
var md;
function preprocessReadme(opts) {
    if (!md) {
        md = new markdown_it_1["default"]({
            html: true,
            linkify: true,
            typographer: true,
            highlight: function (source, lang) {
                if (lang === "svelte") {
                    var regex = new RegExp('"' + opts.name + '"', "g");
                    var modifiedSource = encodeURI(source.replace(regex, '"' + opts.svelte + '"'));
                    var formattedCode = prettier_1["default"].format(source, {
                        parser: "svelte",
                        // @ts-ignore
                        svelteBracketNewLine: true
                    });
                    var svelteCode = prismjs_1["default"].highlight(formattedCode, prismjs_1["default"].languages.svelte, "svelte");
                    return "<pre class=\"language-" + lang + "\" data-svelte=\"" + modifiedSource + "\">{@html `" + svelteCode + "`}</pre>";
                }
                try {
                    return "<pre class=\"language-" + lang + "\">{@html `" + prismjs_1["default"].highlight(source, prismjs_1["default"].languages[lang], lang) + "`}</pre>";
                }
                catch (e) {
                    console.error("Could not highlight language \"" + lang + "\".");
                    return "";
                }
            }
        });
    }
    return {
        markup: function (_a) {
            var content = _a.content, filename = _a.filename;
            if (/node_modules/.test(filename) || !filename.endsWith(".md"))
                return null;
            var script_content = "";
            var style_content = "";
            var result = md.render(content);
            var cursor = 0;
            var ast = compiler_1.parse(result);
            compiler_1.walk(ast, {
                enter: function (node, parent) {
                    if (node.type === "Style") {
                        style_content += result.slice(node.content.start, node.content.end);
                        var replace_style = result.slice(node.start + cursor, node.end + cursor);
                        result = result.replace(replace_style, "");
                        cursor -= replace_style.length;
                    }
                    if (node.type === "Attribute" && node.name === "data-svelte") {
                        var raw_value = node.value[0].raw;
                        var value_1 = decodeURI(raw_value);
                        var value_ast = compiler_1.parse(value_1);
                        var markup = "<div class=\"code-fence\">" + value_1.slice(value_ast.html.start, value_ast.html.end) + "</div>";
                        var replace = result.slice(parent.start + cursor, parent.end + cursor);
                        result = result.replace(replace, markup + replace.replace(raw_value, ""));
                        cursor += markup.length - raw_value.length;
                        compiler_1.walk(value_ast, {
                            enter: function (node) {
                                if (node.type === "Script") {
                                    script_content += value_1.slice(node.content.start, node.content.end);
                                }
                            }
                        });
                    }
                }
            });
            return {
                code: "\n            <script>" + script_content + "</script>\n            <style>\n              :global(.token.language-javascript) { color: #24292e; }\n              :global(.token.language-javascript .function) { color: #005cc5; }\n              :global(.token.language-javascript .string) { color: #032f62; }\n              :global(.token.language-javascript .number) { color: #005cc5; }\n              :global(.token.language-javascript .keyword) { color: #d73a49; }\n              :global(.token.each) { color: #d73a49; }\n              :global(.token.punctuation) { color: #24292e }\n              :global(.token.tag) { color: #22863a; }\n              :global(.token.attr-name) { color: #6f42c1; }\n              :global(.token.operator) { color: #d73a49; }\n              :global(.token.comment) { color: #6a737d; }\n\n              :global(.language-css) { color: #032f62; }\n              :global(.language-css .selector) { color: #22863a; }\n              :global(.language-css .property) { color: #005cc5; }\n\n              .code-fence { padding: 45px 15px; border: 1px solid #eaecef; border-bottom: 0; }\n\n              main {\n                box-sizing: border-box;\n                max-width: 980px;\n                margin: 0 auto;\n                padding: 45px;\n              }\n        \n              @media (max-width: 767px) {\n                main { padding: 15px; }\n              }\n\n              " + style_content + "\n            </style>\n            <main>" + result + "</main>"
            };
        }
    };
}
exports.preprocessReadme = preprocessReadme;
